/*---------------------------------------------------------------------------------------------------*\
|   PROGRAM NAME: dem_etl.sas                                                                         |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Version: Revised 1.0                                                                              |
|   Date: November 26, 2015                                                                           |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|       MSCDM demographic table                                                                       |
|       MSCDM enrollment table                                                                        |
|   Program outputs:                                                                                  |
|       Patkey table                                                                                  |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
\*---------------------------------------------------------------------------------------------------*/

/* 9/21/15 - MR: */
/*  - Dynamically set necessary length of PatKey to avoid truncation/wasted space issues */
/*  - Tighten up valid enrollment criteria to only include Y/N coverages */
/* 10/08/15 - MR: */
/*  - Fix PatKeyLen assignment */
/* 10/17/15 - MR: */
/*  - Modify enrollment criteria to not require at least one Y */
/*  - Modify PatKey assignment so that it is ensured to be sorted in PatID order */
/* 11/2/15 - MR: */
/*    - Modify to set SORTEDBY to PatID */

/* MODIFY TO CALL FMCP NUMLENGTH FUNCTION WITHIN A DATA _NULL_ */

** TASK SUMMARY ;
** ------------- ;
** - Create unique list of patients with at least one span with known coverage values and valid dates;
** - Use enrolled patient list to filter demographic table keeping only records with valid demographics ;
** - Add patKey variable and save records to DPtemp.PatKey ;

proc sql _method ;
  create table ENR_PATS as
    select distinct PatID
      from INDATA.&ENRTABLE
      where (MedCov in ('Y' 'N') and DrugCov in ('Y' 'N')) and
            (. < Enr_Start <= Enr_End) and (Enr_Start <= &DP_MaxDate and Enr_End >= &DP_MinDate) ;
    ;
quit ;

%let PatKeyObs = %DSN_OBS(ENR_PATS);
%let PatKeyLen = %NumLength(&PatKeyObs);
%put When &=PatKeyObs then minimum &=PatKeyLen ;

proc sql _method ;
  create table ENR_PATS_DEM as
    select A.PatID, Birth_Date, Sex
      from INDATA.&DEMTABLE as A
             inner join
           ENR_PATS as B
      on (A.PatID eq B.PatID) and Sex in ('M' 'F') and (Birth_Date > .)
      order by A.PatID
    ;
quit ;

DATA DPtemp.PatKey (sortedby=PatID);
  length PatKey &PatKeyLen..;
  SET ENR_PATS_DEM;
  by PatID;
  PatKey+1;
run;

** Calc number of partitions as the largest of: ;
** -result of the total # of valid patients/200K patients ;
** -result of the largest CDM table size/2G ;
** -the number of years of complete data ;
%let NPatsCriteria = %sysevalf(%DSN_OBS(DPTemp.PatKey)/200000, ceil);
proc sql noprint;
   select ceil(max(filesize)/2000000000) into :FileSizeCriteria
        from dictionary.tables
        where libname eq 'INDATA' and memname in (%QuoteLst(&ST_TableList));
quit;
%let FileSizeCriteria=&FileSizeCriteria;
%put &=DP_NumYears. &=NPatsCriteria &=FileSizeCriteria;
%let N_Partitions = %sysfunc(MAX(&NPatsCriteria., &FileSizeCriteria., &DP_NumYears.));
%put &=N_Partitions;

** Create patient partitions;
%partition_patients ( InDsn=DPTemp.PatKey ,
                      OutDsn=DPTemp.PatKeyPartition ,
                      N_Partitions=&N_Partitions ,
                      PatID=PatID ,
                      OtherKeepVars=PatKey Birth_Date Sex
                    );

** Split PatKey into non-overlaping patient partitions;
%partition_splits ( InDsn=DPTemp.PatKeyPartition ,
                    OutDsn=DPTemp.PartitionPats_ ,
                    numsets=&N_Partitions ,
                    lkpfile=DPTemp.PatKeyPartition ,
                    PatID=PatKey ,
                    SortBy=PatKey
                   );

** Save macro variable N_Partitions into DPtemp.N_Partitions so that other modules may retrieve them;
data DPtemp.N_Partitions;
   length name $32. value 3.;
   name= "N_Partitions"; value= &N_Partitions; output;
run;

** End dem_etl.sas ;
