/*---------------------------------------------------------------------------------------------------*\
|   PROGRAM NAME: incident_rx_summ.sas                                                                |
|     Macro for incident RX Summary Tables summarization                                              |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Version: Revised 1.0                                                                              |
|   Date: September ##, 2015                                                                          |
|                                                                                                     |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Program inputs:                                                                                   |
|       -SAS dataset containing Incident RX Episodes for each lookback period                         |
|       -Utility macro InsertChar                                                                     |
|   Program outputs:                                                                                  |
|       SAS dataset containing summarized episodes for each stratification level                      |
|-----------------------------------------------------------------------------------------------------|
|                                                                                                     |
|   Contact:                                                                                          |
|       Mini-Sentinel Coordinating Center                                                             |
|       info@mini-sentinel.org                                                                        |
\*---------------------------------------------------------------------------------------------------*/

/* MOD: 11/05/15 -MR */
/*                - Replace "crash-merge" with match-merge and check that */
/*                  sum of QTR members equals YEAR members */
/*                - Move helper transpose macro to utility macro file */
/* MOD: 11/15/15 -MR: */
/*              1) Modify program for partitioned approach */
/* MOD: 11/23/15 -MR: */
/*              1) Modify to name work datasets with &DrugCategoryKey prefix */
/* MOD: 01/26/16 -MR */
/*            - JIRA QCA0-5: Ensure all variables on final SAS datasets used for exporting have the */
/*              correct attributes (e.g. length, format, label) that match the specifications */

** TASK SUMMARY ;
** -------------;
** 1- SUMMARIZE EPISODES BY AGEGROUP SEX YEAR LOOKBACK FOR EACH PARTITION;
**     - Summarize measures for Members, EpisodeSpan, Dispensings, DaysSupply;
** 2- SUMMARIZE EPISODES BY AGEGROUP SEX YEAR LOOKBACK QTR FOR EACH PARTITION;
**     - Summarize measures for Members measures only;
** 3- COMBINE PARTITIONED SUMMARIES AND RE-AGGREGATE DATA BY KEYS;
** 4- TRANSPOSE SUMMARIZED MEASURES FROM STEP 3;
**     - Transpose summarized data for each measure by LOOKBACK;
**     - Transpose summarized Members data only by LOOKBACK-QTR;
** 5- FINALIZE;
**     - Merge transposed summaries from steps 4;
**     - Link in DrugCategory description, AgeGroup description, and AgeGroupID;
**     - Finishing touches to create final SAS dataset formatted exactly as required;

%*** Macro for summarizing incident RX episodes for each stratification level;
%macro Incident_Rx_Summ
   ( InFile= ,                                  /* Input file prefix (Note: include libname)*/
     OutFile= ,                                 /* Output file name (Note: include libname)*/
     DrugCategory= ,                            /* Drug Category Name: {GenericName|DrugClass}*/
     DrugCategoryKey= ,                         /* Drug Category Key: {GenericNameKey|DrugClassKey}*/
     DEBUG=N                                    /* Y/N flag for creating extra output (Default is N)*/
   ) ;


  %put *** Display Parameter Values;
  %put _local_;


  /* Set up attributes for DrugCategory */
  %local name_length name_label;
  %if %upcase(&DrugCategory.) eq DRUGCLASS %then %do;
     %let name_length = 70 ;
     %let name_label = Drug Class/Category Name ;
  %end;
  %else %if %upcase(&DrugCategory.) eq GENERICNAME %then %do;
     %let name_length = 30 ;
     %let name_label =  Generic Drug Name ;
  %end;


%put ;
%put ************** BEGIN PARTITION LOOP PROCESSING ******************;
%put ;

  ** Retrieve N_Partitions metadata;
  data _null_;
    set DPTemp.N_Partitions;
    call symputx(Name, Value,'L');
  run;

  %local P;
  %DO P = 1 %to &N_Partitions;  /* BEGIN PARTITION LOOP */

    %put;
    %put *** FOR PARTITION &P , ;
    %put TASK 1- SUMMARIZE EPISODES BY AGEGROUP SEX YEAR LOOKBACK ;
    %put;

    %put *** Summarize episodes by Year for each LookBack;
    %put;

    proc SQL _method;
      CREATE table &DrugCategoryKey._SUMM_BY_YEAR_&P as
        SELECT Sex ,
               AgeEventGroupKey ,
               Year ,
               LookBack ,
               &DrugCategoryKey. ,
               sum(Incident_Episode) as Members ,
               sum(EpiDuration) as EpisodeSpan ,
               sum(EpiNumClaims) as Dispensings ,
               sum(EpiRxSup) as DaysSupply
        FROM &InFile._&P
        GROUP BY &DrugCategoryKey., AgeEventGroupKey, Sex, Year, LookBack
     ;
     quit;



     %put;
     %put *** FOR PARTITION &P , ;
     %put TASK 2- SUMARIZE EPISODES BY AGEGROUP SEX YEAR LOOKBACK QTR ;
     %put;

     %put *** Summarize episodes by Qtr for each LookBack;
     %put;

      proc SQL _method;
        CREATE table &DrugCategoryKey._SUMM_BY_QTR_&P as
          SELECT Sex ,
                 AgeEventGroupKey ,
                 Year ,
                 LookBack ,
                 Qtr ,
                 &DrugCategoryKey. ,
                 sum(Incident_Episode) as Members
          FROM &InFile._&P
          GROUP BY &DrugCategoryKey., AgeEventGroupKey, Sex, Year, LookBack, Qtr
       ;
       quit;

  %END; /* END PARTITION LOOP */
  %put ;

%put ************** END PARTITION LOOP PROCESSING ******************;
%put ;


  %put;
  %put TASK 3-  COMBINE PARTITIONED SUMMARIES AND RE-AGGREGATE DATA BY KEYS;
  %put;

   *** Create temporary view of partitioned subsets aggregated by year;
   DATA &DrugCategoryKey._VIEW_SUMM_BY_YEAR /VIEW=&DrugCategoryKey._VIEW_SUMM_BY_YEAR ;
      SET &DrugCategoryKey._SUMM_BY_YEAR: ;
   RUN;

   *** Re-aggregate partitioned subsets aggregated by YEAR;
   %** DEV Note: order group-by vars from most to least # of distinct values for best performance;
   proc SQL _method;
     CREATE table &DrugCategoryKey._SUMM_BY_YEAR as
       SELECT Sex ,
               AgeEventGroupKey ,
               Year ,
               LookBack ,
               &DrugCategoryKey. ,
               sum(Members) as Members ,
               sum(EpisodeSpan) as EpisodeSpan ,
               sum(Dispensings) as Dispensings ,
               sum(DaysSupply) as DaysSupply
        FROM &DrugCategoryKey._VIEW_SUMM_BY_YEAR
        GROUP BY &DrugCategoryKey., AgeEventGroupKey, Sex, Year, LookBack
     ;
   quit;

   *** Create temporary view of partitioned subsets aggregated by QTR;
   DATA &DrugCategoryKey._VIEW_SUMM_BY_QTR /VIEW=&DrugCategoryKey._VIEW_SUMM_BY_QTR ;
      SET &DrugCategoryKey._SUMM_BY_QTR: ;
   RUN;

   *** Re-aggregate partitioned subsets aggregated by QTR;
   proc SQL _method;
      CREATE table &DrugCategoryKey._SUMM_BY_QTR as
         SELECT Sex ,
                AgeEventGroupKey ,
                Year ,
                LookBack ,
                Qtr ,
                &DrugCategoryKey. ,
                sum(Members) as Members
         FROM &DrugCategoryKey._VIEW_SUMM_BY_QTR
         GROUP BY &DrugCategoryKey., AgeEventGroupKey, Sex, Year, LookBack, Qtr
      ;
   quit;


   %put;
   %put TASK 4-  TRANSPOSE SUMMARIZED MEASURES FROM STEP 3;
   %put;

   %put *** For YEAR aggregates, transpose each measure by LookBack;
   %put;
   %local Measures Measure cm m InDsN Delim ByVars IdVars;

   %let Measures = Members EpisodeSpan Dispensings DaysSupply;
   %let InDsn = &DrugCategoryKey._SUMM_BY_YEAR;
   %let Suffix = Year;
   %let ByVars = %str(&DrugCategoryKey. AgeEventGroupKey Sex Year);
   %let IdVars=LookBack;

   %let cm = %sysfunc(countw(&Measures.));
   %do m = 1 %to &cm;
      %let Measure = %scan(&Measures.,&m);
      %TRANSPOSE
          ( InDsn=&InDsn,
            OutDsn=&DrugCategoryKey._&Measure._&suffix.,
            Delim=&Delim.,
            Prefix=&Measure.,
            By=&ByVars.,
            Id=&IdVars.,
            Var=&Measure.
          );
   %end;

   %put *** For QTR aggregates, transpose each measure by LookBack Qtr;
   %put;

   %let InDsn = &DrugCategoryKey._SUMM_BY_QTR;
   %let Suffix = Qtr;
   %let Delim = Q;
   %let ByVars = %str(&DrugCategoryKey. AgeEventGroupKey Sex Year);
   %let IdVars=LookBack Qtr;

   %let Measure=Members;
   %TRANSPOSE
        ( InDsn=&InDsn,
          OutDsn=&DrugCategoryKey._&Measure._&suffix.,
          Delim=&Delim.,
          Prefix=&Measure.,
          By=&ByVars.,
          Id=&IdVars.,
          Var=&Measure.
        );


   %put;
   %put TASK 5- FINALIZE;
   %put;

   %put *** Merge transposed summaries from step 4;
   %put;

   DATA &DrugCategoryKey._MEASURES_ALL;
      MERGE &DrugCategoryKey._Members_Year (in=a)
            &DrugCategoryKey._EpisodeSPAN_Year (in=b)
            &DrugCategoryKey._Dispensings_Year (in=c)
            &DrugCategoryKey._DaysSupply_Year (in=d)
            &DrugCategoryKey._Members_QTR (in=e);
      BY &DrugCategoryKey. AgeEventGroupKey Sex Year;
      ** Verify there is a matching record from each input dataset;
      if NOT (a & b & c & d & e) then abort cancel;

      ** Replace numeric missing values with 0;
      array M(*) _NUMERIC_;
      do i=1 to dim(M);
         if M(i) eq . then M(i)=0;
      end;
      %if %upcase(&DEBUG.) eq Y %then %do;
        ** Verify quarterly member counts equal year member counts;
        if (sum(of Members90Q:) ne Members90 ) then abort cancel;
        if (sum(of Members180Q:) ne Members180 ) then abort cancel;
        if (sum(of Members270Q:) ne Members270 ) then abort cancel;
      %end;
      drop i;
   RUN;

   %put *** Link in Descriptions;
   %put;

   *** Link in &DrugCategory description;

   proc SQL _method;
      CREATE table &DrugCategoryKey._MEASURES_LINKED as
         SELECT rx.*, lk.&DrugCategory. length=&name_length.
            FROM &DrugCategoryKey._MEASURES_ALL as rx
                    INNER JOIN
                 DPtemp.&DrugCategory.lookup as lk
            ON rx.&DrugCategoryKey eq lk.&DrugCategoryKey
            ORDER BY AgeEventGroupKey, Sex, Year, &DrugCategory.
          ;
   quit;

   *** Link in AgeGroup description and AgeGroupID;

   DATA &DrugCategoryKey._MEASURES_LINKED;
     SET &DrugCategoryKey._MEASURES_LINKED ;
     Age_Group = put(AgeEventGroupKey,$AgeGroupID_to_Value.);
     length Age_Group_ID 3.;
     Age_Group_ID = input(put(AgeEventGroupKey,$AgeGroupID_to_True_ID.),2.) ;
     drop AgeEventGroupKey &DrugCategoryKey.;
   RUN ;

   %put *** Finishing touches to create final SAS dataset formatted exactly as required;
   %put;

   proc SORT data=&DrugCategoryKey._MEASURES_LINKED (rename=(Year=Period));
     BY Age_Group_ID Sex Period &DrugCategory.;
   run;

   /* Create sorted order for Keys */
   %local Keys;
   %let Keys= Age_Group Sex Period &DrugCategory;

   /* Create sorted order for Measures */
   %local LookBacks;
   %let LookBacks = 90 180 270;

   %local Measures cl l lv ;
   %let cl = %sysfunc(countw(&LookBacks.));
   %let Measures = ;
   %do l = 1 %to &cl;
      %let lv = %scan(&LookBacks.,&l.);
      %let Measures = %cmpres(&Measures. Members&lv. EpisodeSpan&lv. Dispensings&lv. DaysSupply&lv.
                              Members&lv.Q1 Members&lv.Q2 Members&lv.Q3 Members&lv.Q4);
   %end;
   %*put &=Measures;

   /* Note: Use attribute-statements "Jiu-Jitsu" in order to: */
   /*   (a) force variable order (a nice side-effect of placing between DATA and SET statements) */
   /*   (b) set default numeric format length                     */
   /*   (c) override format length for numeric variable Age_ID */
   /*   (d) use default format length for character vars which default to physical var length */

  /* Dev/QC Note: Character formats are now explicitly set to comply with specifications. */
  /*              Likewise, variable labels are now assigned to comply with specifications. */

   *** Output Final SAS Dataset;
   DATA &OUTFILE.;
      informat &Keys. &Measures. Age_Group_ID;
      SET &DrugCategoryKey._MEASURES_LINKED;

      format Age_Group $5. &DrugCategory. $&name_length.. Period $4. Sex $1.;
      format _NUMERIC_ 15. ;
      format Age_Group_ID 2.;
      informat _ALL_ ;

      label Age_Group       = 'Age Group'
            Sex             = 'Sex'
            Period          = 'Year'
            &DrugCategory   = "&name_label."
            %do l = 1 %to &cl;
              %let lv = %scan(&LookBacks.,&l.);
              Members&lv       = "Members with no same drug dispensing in &lv-day lookback prior to index date"
              Dispensings&lv   = "# of Dispensings for &lv-day lookback"
              DaysSupply&lv    = "# of total days supply for all episodes for &lv-day lookback"
              EpisodeSpan&lv   = "# of total length in days of all episodes for &lv-day lookback"
              Members&lv.Q1    = "# of index dates in Period/Q1 for &lv-day lookback"
              Members&lv.Q2    = "# of index dates in Period/Q2 for &lv-day lookback"
              Members&lv.Q3    = "# of index dates in Period/Q3 for &lv-day loolback"
              Members&lv.Q4    = "# of index dates in Period/Q4 for &lv-day lookback"
            %end;
            Age_Group_ID    = 'ID'
            ;
   RUN;

%mend Incident_Rx_Summ;

* End of incident_rx_summ.sas;